// $Id: rc5.c,v 1.2 2005/05/08 08:53:37 pio Exp $

/*									tab:4
 * "Copyright (c) 2000-2003 The Regents of the University  of California.  
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs and the author appear in all copies of this software.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF
 * CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS."
 *
 * Copyright (c) 2002-2003 Intel Corporation
 * All rights reserved.
 *
 * This file is distributed under the terms in the attached INTEL-LICENSE     
 * file. If you do not find these files, copies can be found by writing to
 * Intel Research Berkeley, 2150 Shattuck Avenue, Suite 1300, Berkeley, CA, 
 * 94704.  Attention:  Intel License Inquiry.
 */

/* Authors: Naveen Sastry
 * Date:    9/29/02
 */

/**
 * @author Naveen Sastry
 */

#include <stdio.h>
#include "crypto.h"
#include "CryptoPrimitives.h"
//includes crypto;

#define RC5_32_P	0xB7E15163L
#define RC5_32_Q	0x9E3779B9L
#define RC5_32_MASK	0xffffffffL

#define RC5_ROUNDS 12

#define rotl32(a,b) fastrol32((a), (b))
#define rotr32(a,b) fastror32((a), (b))

//module RC5M {
//  provides interface BlockCipher;
//  provides interface BlockCipherInfo;
//}
//implementation
//{

  #define result_t uint32_t
  #define SUCCESS 1
  #define FAIL 0

  uint32_t rotate_left32(uint32_t a, uint8_t n) {
	return (((a)<<(n&0x1f))|(((a)&0xffffffff)>>(32-(n&0x1f)))); 
  }

  uint32_t rotate_right32(uint32_t a, uint8_t n) {
	return (((a)<<(32-(n&0x1f)))|(((a)&0xffffffff)>>(n&0x1f)));
  }

  uint32_t fastrol32(uint32_t a, uint8_t n) {
    n = n & 0x1f;
    if ((n) == 0) {
	return a;
    } else if ((n) >= 1 && (n) <= 5) {
        return rol32(a, (n));
    } else if ((n) >= 6 && (n) <= 7) {
        a = brol1(a);
        return ror32(a, 8-(n));
    } else if ((n) >= 8 && (n) <= 12) {
        a = brol1(a);
        return ror32(a, (n)-8);
    } else if ((n) >= 13 && (n) <= 16) {
        a = brol2(a);
        return ror32(a, 16-(n));
    } else if ((n) >= 17 && (n) <= 20) {
        a = brol2(a);
        return rol32(a, (n)-16);
    } else if ((n) >= 21 && (n) <= 24) {
        a = brol3(a);
        return ror32(a, 24-(n));
    } else if ((n) >= 25 && (n) <= 28) {
        a = brol3(a);
        return ror32(a, (n)-24);
    } else if ((n) >= 29 && (n) <= 31) {
        return ror32(a, 32-(n));
    } else {
        printf("shouldn't happen!\n");
	printf("n is %u\n", n);
	exit(1);
    }
  }


  // 2 * (ROUNDS +1) * 4 
  // 2 * 13 * 4 = 104 bytes
   typedef struct RC5Context {
    uint32_t skey [2* (RC5_ROUNDS + 1) ];
   } RC5Context;
   
   enum { BSIZE = 8 };
   
  result_t setupKey (CipherContext * context, uint8_t * key);

  void dumpBuffer (char * bufName, uint8_t * buf, uint8_t size)
    {
#ifdef O
      uint8_t i = 0;
      // fixme watch buffer overrun
      char tmp[512];
      for (; i < size; i++) {
        sprintf (tmp + i * 3, "%2x ", (char)buf[i] & 0xff);
      }
      dbg(DBG_CRYPTO, "%s: {%s}\n", bufName, tmp);
#endif
    }
 
  /**
   * Initialize the BlockCipher. 
   *
   * @return Whether initialization was successful. The command may be
   *         unsuccessful if the key size or blockSize are not valid for the
   *         given cipher implementation. 
   */
  result_t init(CipherContext * context, uint8_t blockSize,
                                    uint8_t keySize, uint8_t * key)
    {
      // 8 byte blocks only
      if (blockSize != BSIZE) {
        return FAIL;
      }
      return setupKey (context, key);
    }

  /**
   * Encrypts a single block (of blockSize) using the key in the keySize.
   *
   * PROLOGUE: 24 cycles
   * INIT:     48 cycles
   * LOOP:   1680 cycles (12 + fastrol [= 42] + 16) * 2 * RC5_ROUNDS
   * CLOSE:    24 cycles
   * =====================
   *         1776 cycles (avg case)
   * 
   * @param plainBlock a plaintext block of blockSize
   * @param cipherBlock the resulting ciphertext block of blockSize
   *
   * @return Whether the encryption was successful. Possible failure reasons
   *         include not calling init(). 
   */
  result_t encrypt(CipherContext * context,
                                       uint8_t * block, uint8_t * cipherBlock)
    {
      register uint32_t l;
      register uint32_t r;
      register uint32_t *s = ((RC5Context*) context->context)->skey;
      uint8_t i, tmp;
      c2l(block, l);
      block += 4;
      c2l(block, r);
      l += *s++;
      r += *s++;
      for (i = RC5_ROUNDS; i > 0; i--) {
        l ^= r;   tmp = r; tmp &= 0x1f; rotl32(l, tmp);   l += *s++;
        r ^= l;   tmp = l; tmp &= 0x1f; rotl32(r, tmp);   r += *s++;
      }
      l2c(l, cipherBlock);
      cipherBlock += 4;
      l2c(r, cipherBlock);
      return SUCCESS;
    }

  /**
   * Decrypts a single block (of blockSize) using the key in the keySize. Not
   * all ciphers will implement this function (since providing encryption
   * is a useful primitive). 
   *
   * @param cipherBlock a ciphertext block of blockSize
   * @param plainBlock the resulting plaintext block of blockSize
   *
   * @return Whether the decryption was successful. Possible failure reasons
   *         include not calling init() or an unimplimented decrypt function.
   */
  result_t decrypt(CipherContext * context,
                                       uint8_t * cipherBlock,
                                       uint8_t * plainBlock)
    {
      register uint32_t l;
      register uint32_t r;
      register uint32_t *s = ((RC5Context*) context->context)->skey +
        (2 * RC5_ROUNDS) + 1;
      uint8_t i, tmp;

      c2l(cipherBlock, l);
      cipherBlock += 4;
      c2l(cipherBlock, r);
      for (i = RC5_ROUNDS; i> 0; i--) {
        r -= *s--;   tmp = l;  tmp &= 0x1f; rotr32(r, tmp);   r ^= l;
        l -= *s--;   tmp = r;  tmp &= 0x1f; rotr32(l, tmp);   l ^= r;
      }
      r -= *s--;
      l -= *s;
      l2c(l, plainBlock);
      plainBlock += 4;
      l2c(r, plainBlock);
      return SUCCESS;
    }

  /**
   * Performs the key expansion on the real secret.
   *
   * @param secret key
   */
  result_t setupKey (CipherContext * context, uint8_t * key)
    {
      uint32_t *L,l,A,B,*S,k;
      uint8_t ii,jj, m;
      int8_t i;
      uint8_t tmp[8];
      S= ((RC5Context*)context->context)->skey;

      dumpBuffer ("RC5M:setupKey K", (uint8_t *)key, 8);      
      c2l(key,l);
      L = (uint32_t *) tmp;
      L[0]=l;
      key += 4;
      c2l(key,l);
      L[1]=l;
      S[0]=RC5_32_P;
      dumpBuffer ("RC5M:setupKey L", (uint8_t *)L, 8);      
      for (i=1; i< 2 * RC5_ROUNDS + 2; i++) {
        S[i] = (S[i-1] + RC5_32_Q);
        /*        sum =(*S+RC5_32_Q)&RC5_32_MASK;
                  S++;
                  *S = sum;
                  */
      }
      dumpBuffer ("RC5M: setupKey S", (uint8_t *)S, 2 * (RC5_ROUNDS +1) * 4);
      ii=jj=0;
      A=B=0;
      S= ((RC5Context*)context->context)->skey;
      for (i=3*(2*RC5_ROUNDS + 2) - 1; i>=0; i--) {
        k=(*S+A+B)&RC5_32_MASK;
        rotl32((k), (3));
        A=*S= k;
        S++;
        m=((char)(A+B)) & 0x1f;
        k=(*L+A+B)&RC5_32_MASK;
        rotl32((k), (m));
        B=*L= k;
        if (++ii >= 2*RC5_ROUNDS+2) {ii=0; S= ((RC5Context*)context->context)->skey; }
        jj ^= 4;
        L = (uint32_t *) (&tmp[jj]);
      }
      dumpBuffer ("RC5M: setupKey S",
                  (uint8_t*)((RC5Context*)context->context)->skey,
                  2 * (RC5_ROUNDS + 1) * 4);
      return SUCCESS;
    }

  /**
   * Returns the preferred block size that this cipher operates with. It is
   * always safe to call this function before the init() call has been made.
   *
   * @return the preferred block size for this cipher. In the case where the
   *         cipher operates with multiple block sizes, this will pick one
   *         particular size (deterministically).
   */
  uint8_t getPreferredBlockSize()
    {
      return BSIZE;
    }

#define KEYSIZE 16
typedef uint32_t WORD;

/*
   Testbench for crytography algorithm
 */
int main(int argc, char **argv)
{ 
  WORD i, j;
  //WORD PlainText1[2], PlainText2[2];
  uint8_t PlainText1[8], PlainText2[8];
  uint8_t CryptoText[8];
  //WORD CryptoText[2] = {0,0};
  // time_t t0, t1;
  char *keystr;
  CipherContext context;

  if (argc != 2) {
        printf("Wrong number of arguments!\n"); 
        exit(1);
  }

  keystr = argv[1];

  init(&context, 8, KEYSIZE, keystr);


  if (sizeof(WORD)!=4) 
    printf("RC5 error: WORD has %d bytes.\n",sizeof(WORD));

  memset(PlainText1, 0, 8);
  memset(PlainText2, 0, 8);
  memset(CryptoText, 0, 8);

  printf("RC5-32/12/16 examples:\n");
      PlainText1[0] = CryptoText[0]; 
      PlainText1[1] = CryptoText[1]; 
      //for (j=0; j < KEYSIZE; j++) 
      //   SetKey((unsigned char)keystr[j]);
      /* Setup, encrypt, and decrypt */
      //setup();  

      encrypt(&context, PlainText1,CryptoText);  
      decrypt(&context, CryptoText,PlainText2);

      uint32_t *ptxt1_left = PlainText1;
      uint32_t *ptxt1_right = PlainText1 + 4;
      uint32_t *ctxt_left = CryptoText;
      uint32_t *ctxt_right = CryptoText + 4;
      uint32_t *ptxt2_left = PlainText2;
      uint32_t *ptxt2_right = PlainText2 + 4;


      /* Print out results, checking for decryption failure */
      printf("\n   plaintext %.8lX %.8lX  --->  ciphertext %.8lX %.8lX\n",
             *ptxt1_left, *ptxt1_right, *ctxt_left, *ctxt_right);
      printf("\n   ciphertext %.8lX %.8lX  --->  plaintext %.8lX %.8lX\n",
             *ctxt_left, *ctxt_right, *ptxt2_left, *ptxt2_right);
      if (PlainText1[0]!=PlainText2[0] || PlainText1[1]!=PlainText2[1]) 
        printf("Decryption Error!");
//  time (&t0);
//  for (i=1;i<5000000;i++)
//     encrypt(CryptoText,CryptoText);
//  time (&t1);
//  printf ("\n   Time_t for 5000000 blocks:  %ld \n", t1-t0);
 return(0);
}

    


