import java.io.*;
import java.util.*;

/**
 * This is the main class that opens the file and feeds it to the parsing
 * and verifying machinery.
 */
public class Verifier {
    protected AsmNode program;

    public Verifier(Reader in) {
	 program = AsmNode.makeAsmNode(in);
    }

    public String toString() {
	if (program != null)
	    return program.toString();
	else
	    return "(could not parse program)";
    }

    public void verify() {
	Map<String, AsmNode> globals = program.getGlobals();
	if (globals.isEmpty())
	    System.err.println("warning: no global symbols found");
	for (Map.Entry<String, AsmNode> entryPoint : globals.entrySet()) {
	    Configuration c = new Configuration(entryPoint.getValue());
	    System.out.println("Checking entry point " + entryPoint.getKey());
	    c.simulate();
	    System.out.println();
	}
    }

    protected static void usage() {
	System.err.println("usage: Verifier [-d] filename");
	System.exit(1);
    }

    public static void main(String args[]) {
	String fileName = null;
	if (args.length == 2) {
	    if (!args[0].equals("-d"))
		usage();
	    Debug.setDebug(2);
	    fileName = args[1];
	} else if (args.length == 1) {
	    fileName = args[0];
	} else {
	    usage();
	}
	try {
	    Verifier v = new Verifier(new FileReader(fileName));
	    if (Debug.getDebug() >= 2)
		Debug.println(2, v.toString());
	    v.verify();
	} catch (FileNotFoundException e) {
	    System.err.println(fileName + ": not found");
	    System.exit(1);
	}
    }
}
